module bimodal_dsge
  use mkl95_precision, only: wp => dp

  use prior
  use filter
!  use strid_walentin

  implicit none

  character(len=*), parameter :: mname = 'bimodal-dsge'

  character(len=*), parameter :: priorfile = '~/code/fortran/models/bimodal-dsge/flat_prior.txt'
  character(len=*), parameter :: priorfile2 = '~/code/fortran/models/bimodal-dsge/flat_prior.txt'
  character(len=*), parameter :: datafile = '~/code/fortran/models/bimodal-dsge/data.txt'
  character(len=*), parameter :: transfile = '~/code/fortran/models/bimodal-dsge/trspec.txt'
  character(len=*), parameter :: initfile = '~/code/fortran/models/bimodal-dsge/parasim_from_beta_prior.txt'
  character(len=*), parameter :: initwt = ''

  character(len=*), parameter :: varfile = 'models/bimodal-dsge/smc_var.txt'

  integer, parameter :: neq = 2, neta = 2, neps = 1, nobs = 200, npara = 2, ns = 2, ny = 1
  real(wp), parameter :: REALLY_NEG = -10.0_wp**10

  integer, parameter :: t0 = 0
  real(wp) :: phi1, phi2, phi3
  real(wp) :: YY(ny, nobs)

  ! prior stuff
  integer :: pshape(npara), pmask(npara)
  real(wp) :: pmean(npara), pstdd(npara), pfix(npara)

  integer :: pshape2(npara), pmask2(npara)
  real(wp) :: pmean2(npara), pstdd2(npara), pfix2(npara)

  ! prtr
  real(wp) :: trspec(4, npara)

contains

  include '../../base/helper_functions.f90'

  subroutine sysmat(p0, TT, RR, QQ, DD, ZZ, HH, info)

    real(wp), intent(in) :: p0(npara)

    real(wp), intent(out) :: TT(ns, ns), RR(ns, neps), QQ(neps, neps), DD(ny), ZZ(ny, ns), HH(ny, ny)
    integer, intent(out) :: info


    phi1 = p0(1)**2
    phi2 = 1.0_wp - phi1
    phi3 = -p0(1) * p0(2) + phi2

    ! state transition
    TT(1,:) = (/phi1, 0.0_wp/)
    TT(2,:) = (/phi3, phi2/)
    RR(:,1) = (/1.0_wp, 0.0_wp/)
    QQ = 1.0_wp
    
    ZZ(1,:) = (/ 1.0_wp, 1.0_wp /)
    DD = 0.0_wp
    HH = 0.0_wp
    
    info = 1
  end subroutine sysmat

  function pmsv result (para)
    real(wp) :: para(npara)
    
    para = (/0.2_wp, 0.2_wp/)
  end function pmsv
end module bimodal_dsge
